"""
AUTO-GENERATED BY Testing Kit v1

Tests for the population factory registry discovery behaviour.
"""
from pyparticle.population.factory.registry import discover_population_types


def test_discover_population_types_returns_dict():
    types = discover_population_types()
    assert isinstance(types, dict)
    # If any types present, they should be callables
    for k, v in types.items():
        assert callable(v)
