"""
AUTO-GENERATED BY Testing Kit v1

Tests for the viz plotter discovery registry.
"""
from pyparticle.viz.factory.registry import discover_plotter_types


def test_discover_plotter_types_returns_dict():
    types = discover_plotter_types()
    assert isinstance(types, dict)
    for k, v in types.items():
        assert callable(v)
